package javax.security.auth.kerberos;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.security.PermissionCollection;
import java.io.Serializable;
import java.security.Permission;

public final class ServicePermission extends Permission implements Serializable
{
    private static final long serialVersionUID = -1227585031618624935L;
    private static final int INITIATE = 1;
    private static final int ACCEPT = 2;
    private static final int ALL = 3;
    private static final int NONE = 0;
    private transient int mask;
    private String actions;
    
    public ServicePermission(String name, String s) {
        super(name);
        this.init(name, getMask(s));
    }
    
    private void init(String s, int mask) {
        if (s == null) {
            throw new NullPointerException("service principal can't be null");
        }
        if ((mask & 0x3) != mask) {
            throw new IllegalArgumentException("invalid actions mask");
        }
        this.mask = mask;
        return;
    }
    
    @Override
    public boolean implies(Permission permission) {
        if (!(permission instanceof ServicePermission)) {
            return false;
        }
        ServicePermission servicePermission = (ServicePermission)permission;
        return (this.mask & servicePermission.mask) == servicePermission.mask && this.impliesIgnoreMask(servicePermission);
    }
    
    boolean impliesIgnoreMask(ServicePermission servicePermission) {
        return this.getName().equals("*") || this.getName().equals(servicePermission.getName()) || (servicePermission.getName().startsWith("@") && this.getName().endsWith(servicePermission.getName()));
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof ServicePermission)) {
            return false;
        }
        ServicePermission servicePermission = (ServicePermission)o;
        return (this.mask & servicePermission.mask) == servicePermission.mask && this.getName().equals(servicePermission.getName());
    }
    
    @Override
    public int hashCode() {
        return this.getName().hashCode() ^ this.mask;
    }
    
    private static String getActions(int n) {
        StringBuilder sb = new StringBuilder();
        int n2 = 0;
        if ((n & 0x1) == 0x1) {
            if (n2 != 0) {
                sb.append(',');
            }
            else {
                n2 = 1;
            }
            sb.append("initiate");
        }
        if ((n & 0x2) == 0x2) {
            if (n2 != 0) {
                sb.append(',');
            }
            sb.append("accept");
        }
        return sb.toString();
    }
    
    @Override
    public String getActions() {
        if (this.actions == null) {
            this.actions = getActions(this.mask);
        }
        return this.actions;
    }
    
    @Override
    public PermissionCollection newPermissionCollection() {
        return new KrbServicePermissionCollection();
    }
    
    int getMask() {
        return this.mask;
    }
    
    private static int getMask(String s) {
        if (s == null) {
            throw new NullPointerException("action can't be null");
        }
        if (s.equals("")) {
            throw new IllegalArgumentException("action can't be empty");
        }
        int n = 0;
        char[] charArray = s.toCharArray();
        if (charArray.length == 1 && charArray[0] == '-') {
            return n;
        }
        int n3;
        for (int i = charArray.length - 1; i != -1; i -= n3) {
            int n2;
            while (i != -1 && ((n2 = charArray[i]) == ' ' || n2 == '\r' || n2 == '\n' || n2 == '\f' || n2 == '\t')) {
                --i;
            }
            if (i >= 7 && (charArray[i - 7] == 'i' || charArray[i - 7] == 'I') && (charArray[i - 6] == 'n' || charArray[i - 6] == 'N') && (charArray[i - 5] == 'i' || charArray[i - 5] == 'I') && (charArray[i - 4] == 't' || charArray[i - 4] == 'T') && (charArray[i - 3] == 'i' || charArray[i - 3] == 'I') && (charArray[i - 2] == 'a' || charArray[i - 2] == 'A') && (charArray[i - 1] == 't' || charArray[i - 1] == 'T') && (charArray[i] == 'e' || charArray[i] == 'E')) {
                n3 = 8;
                n |= 0x1;
            }
            else if (i >= 5 && (charArray[i - 5] == 'a' || charArray[i - 5] == 'A') && (charArray[i - 4] == 'c' || charArray[i - 4] == 'C') && (charArray[i - 3] == 'c' || charArray[i - 3] == 'C') && (charArray[i - 2] == 'e' || charArray[i - 2] == 'E') && (charArray[i - 1] == 'p' || charArray[i - 1] == 'P') && (charArray[i] == 't' || charArray[i] == 'T')) {
                n3 = 6;
                n |= 0x2;
            }
            else {
                throw new IllegalArgumentException("invalid permission: " + s);
            }
            for (int n4 = 0; i >= n3 && n4 == 0; --i) {
                switch (charArray[i - n3]) {
                    case ',': {
                        n4 = 1;
                        break;
                    }
                    case '\t':
                    case '\n':
                    case '\f':
                    case '\r':
                    case ' ': {
                        break;
                    }
                    default: {
                        throw new IllegalArgumentException("invalid permission: " + s);
                    }
                }
            }
        }
        return n;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        if (this.actions == null) {
            this.getActions();
        }
        objectOutputStream.defaultWriteObject();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.init(this.getName(), getMask(this.actions));
    }
}
